package main

import (
	"context"
	"fmt"
	"image"
	"log"

	"git.milar.in/milarin/hypr"
	"git.milar.in/milarin/slices"
)

func main() {
	client, err := hypr.GetDefaultClient()
	if err != nil {
		panic(err)
	}

	events, err := client.Subscribe(context.Background(), hypr.EventTypeFullscreen)
	if err != nil {
		panic(err)
	}

	if err := RestoreWindowPositions(client); err != nil {
		panic(err)
	}

	for event := range events {
		if len(event.Data) != 1 || event.Data[0] != "0" {
			continue
		}

		if err := RestoreWindowPositions(client); err != nil {
			panic(err)
		}
	}
}

func RestoreWindowPositions(client *hypr.Client) error {
	monitors, err := client.GetMonitors()
	if err != nil {
		return err
	}

	monitorBoundsByID := slices.ToMap(monitors, func(m *hypr.Monitor) (int, image.Rectangle) {
		return m.ID, image.Rect(m.X, m.Y, m.X+m.Width, m.Y+m.Height)
	})

	windows, err := client.GetWindows()
	if err != nil {
		return err
	}

	for _, window := range windows {
		// ignore all special workspaces because they can't be moved between monitors
		if window.Workspace.ID < 0 {
			continue
		}

		windowBounds := image.Rect(
			window.At[0],
			window.At[1],
			window.At[0]+window.Size[0],
			window.At[1]+window.Size[1])

		monitorBounds := monitorBoundsByID[window.MonitorID]
		if windowBounds.In(monitorBounds) {
			continue
		}

		// retrieve the bounds of the monitor in which the window is actually drawn.
		// This can be the default value of Rect(-1, -1, -1, -1)
		// if the windows is out of bounds for any monitor
		boundsOfActualMonitor := image.Rect(-1, -1, -1, -1)
		for _, monitorBounds := range monitorBoundsByID {
			if windowBounds.In(monitorBounds) {
				boundsOfActualMonitor = monitorBounds
				break
			}
		}

		// calculate new position based on the position of the window and its actual monitor
		diff := windowBounds.Min.Sub(boundsOfActualMonitor.Min)
		shouldPos := monitorBounds.Min.Add(diff)

		// use coords [0,0] respective to the wanted monitor
		// if window is completely OOB
		if boundsOfActualMonitor == image.Rect(-1, -1, -1, -1) {
			shouldPos = monitorBounds.Min
		}

		cmd := fmt.Sprintf(
			"movewindowpixel exact %d %d,address:%s",
			shouldPos.X,
			shouldPos.Y,
			window.Address)

		if err := client.DispatchExpectOK(cmd); err != nil {
			log.Printf("invalid position could NOT be restored successfully for window (class: '%s' | title: '%s')\n", window.Class, window.Title)
			continue
		}

		log.Printf("invalid position restored for window (class: '%s' | title: '%s')\n", window.Class, window.Title)
	}

	return nil
}
